local L = PitBull:L("PitBull-Spark")

local PitBull_Spark = PitBull:NewModule("Spark", "AceEvent-2.0")

local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

local GetTime = GetTime
local UnitMana = UnitMana
local UnitPowerType = UnitPowerType

PitBull_Spark:RegisterPitBullChildFrames('powerSpark')

local powerSpark, manaBar, energy_lasttick, current_energy, spellcast_finish_time, channeling, power, manaOnUpdate, energyOnUpdate

local energyticklength = 2
local fiveseclength = 5

function manaOnUpdate(frame)
	frame:ClearAllPoints()
	local timediff = GetTime() - spellcast_finish_time
	if timediff > fiveseclength then
		frame:Hide()
		return frame:SetScript('OnUpdate', nil)
	else
		frame:SetPoint('LEFT', manaBar, 'LEFT', (timediff / fiveseclength) * manaBar:GetWidth(), 0)
	end
end

function energyOnUpdate(frame)
	frame:ClearAllPoints()
	local timediff = GetTime() - energy_lasttick
	if timediff > energyticklength then
		energy_lasttick = energy_lasttick + energyticklength
	end
	frame:SetPoint('LEFT', manaBar, 'LEFT', (timediff / energyticklength) * manaBar:GetWidth(), 0)
end

function PitBull_Spark:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("Spark")
	PitBull:RegisterDefaults("Spark", "profile", {
		color = {1, 1, 1, 0.3},
		enable = true,
	})
end

function PitBull_Spark:OnEnable()
	power = UnitPowerType('player')
	current_energy = 0
	if self.db.profile.enable then
		self:DoRegistrations()
	end
end

function PitBull_Spark:DoRegistrations()
	local _, class = UnitClass('player')
	if class == "ROGUE" then
		self:RegisterEvent("UNIT_ENERGY")
		self.UNIT_DISPLAYPOWER = nil
		self.UNIT_SPELLCAST_CHANNEL_STOP = nil
		self.UNIT_SPELLCAST_CHANNEL_START = nil
		self.UNIT_SPELLCAST_SUCCEEDED = nil
	elseif class == "DRUID" then
		self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
		self:RegisterEvent("UNIT_ENERGY")
		self:RegisterEvent("UNIT_DISPLAYPOWER")
	elseif class ~= "WARRIOR" then
		self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
		self.UNIT_ENERGY = nil
		self.UNIT_DISPLAYPOWER = nil
	else
		self.UNIT_ENERGY = nil
		self.UNIT_DISPLAYPOWER = nil
		self.UNIT_SPELLCAST_CHANNEL_STOP = nil
		self.UNIT_SPELLCAST_CHANNEL_START = nil
		self.UNIT_SPELLCAST_SUCCEEDED = nil
		self.OnPopulateUnitFrame = nil
		self.OnUpdateLayout = nil
		self.OnClearUnitFrame = nil
	end
end

function PitBull_Spark:UNIT_ENERGY(unit)
	if unit ~= 'player' then
		return
	end
	local newenergy = UnitMana('player')
	if newenergy > current_energy and power == 3 then
		energy_lasttick = GetTime()
		powerSpark:Show()
		powerSpark:SetHeight(manaBar:GetHeight() * 1.5)
		powerSpark.texture:SetHeight(manaBar:GetHeight() * 1.5)
		powerSpark:SetScript('OnUpdate', energyOnUpdate)
	end
	current_energy = newenergy
end

function PitBull_Spark:UNIT_DISPLAYPOWER(unit)
	if unit == 'player' then
		power = UnitPowerType('player')
		if power == 3 then
			-- kitty form
			spellcast_finish_time = nil
			current_energy = UnitMana('player')
			powerSpark:Hide()
			powerSpark:SetScript('OnUpdate', nil)
		elseif power == 0 then
			-- pretty blue bar, make sure it knows we just cast a spell shifting out (that does stop regen, right?)
			energy_lasttick = nil
			self:UNIT_SPELLCAST_SUCCEEDED('player')
		else
			-- bare fite
			spellcast_finish_time = nil
			energy_lasttick = nil
			powerSpark:Hide()
			powerSpark:SetScript('OnUpdate', nil)
		end
	end
end

function PitBull_Spark:UNIT_SPELLCAST_SUCCEEDED(unit)
	if unit == 'player' and power == 0 and not channeling then
		spellcast_finish_time = GetTime()
		powerSpark:Show()
		powerSpark:SetHeight(manaBar:GetHeight() * 1.5)
		powerSpark.texture:SetHeight(manaBar:GetHeight() * 1.5)
		powerSpark:SetScript('OnUpdate', manaOnUpdate)
	end
end

function PitBull_Spark:UNIT_SPELLCAST_CHANNEL_START(unit)
	if unit == 'player' and power == 0 then
		channeling = true
		powerSpark:Hide()
		powerSpark:SetScript('OnUpdate', nil)
	end
end

function PitBull_Spark:UNIT_SPELLCAST_CHANNEL_STOP(unit)
	if unit == 'player' and power == 0 then
		channeling = false
		spellcast_finish_time = GetTime()
		powerSpark:Show()
		powerSpark:SetHeight(manaBar:GetHeight() * 1.5)
		powerSpark.texture:SetHeight(manaBar:GetHeight() * 1.5)
		powerSpark:SetScript('OnUpdate', manaOnUpdate)
	end
end

function PitBull_Spark:OnPopulateUnitFrame(unit, frame)
	if unit == 'player' and self.db.profile.enable and frame.manaBar and select(2, UnitClass('player')) ~= "WARRIOR" then
		powerSpark = newFrame("Frame", frame)
		frame.powerSpark = powerSpark
		
		local texture = newFrame("Texture", powerSpark, "OVERLAY")
		frame.powerSpark.texture = texture
		texture:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
		texture:SetVertexColor(unpack(self.db.profile.color))
		texture:SetBlendMode('ADD')
		texture:Show()
		
		manaBar = frame.ManaBar
	end
end

function PitBull_Spark:OnUpdateLayout(unit, frame)
	if unit == 'player' then
		manaBar = frame.manaBar
		
		powerSpark.texture:SetPoint('CENTER', powerSpark, 'LEFT')
		
		powerSpark:SetWidth(20)
		powerSpark.texture:SetWidth(20)
		
		local script = powerSpark:GetScript('OnUpdate')
		if script then
			powerSpark.texture:Show()
			powerSpark:Show()
			powerSpark:SetHeight(manaBar:GetHeight() * 1.5)
			powerSpark.texture:SetHeight(manaBar:GetHeight() * 1.5)
			powerSpark:SetScript('OnUpdate', script)
		else
			powerSpark:Hide()
		end
	end
end

function PitBull_Spark:OnClearUnitFrame(unit, frame)
	if unit == 'player' and frame.powerSpark then
		frame.powerSpark.texture = delFrame(frame.powerSpark.texture)
		frame.powerSpark = delFrame(frame.powerSpark)
		powerSpark = nil
	end
end

local function enabled_get()
	return PitBull_Spark.db.profile.enable
end
local function enabled_set(v)
	PitBull_Spark.db.profile.enable = v
	if select(2, UnitClass('player')) == "WARRIOR" then
		return
	end
	if v then
		if not powerSpark then
			PitBull_Spark:OnPopulateUnitFrame('player', PitBull:GetUnitFrame('player'))
		end
		PitBull_Spark:DoRegistrations()
	else
		PitBull_Spark:UnregisterAllEvents()
		if powerSpark then
			powerSpark:Hide()
			powerSpark:SetScript('OnUpdate', nil)
		end
	end
end
local function color_get()
	return unpack(PitBull_Spark.db.profile.color)
end
local function color_set(r,g,b,a)
	PitBull_Spark.db.profile.color = {r,g,b,a}
	if powerSpark then
		powerSpark.texture:SetVertexColor(r,g,b,a)
	end
end

PitBull_Spark:RegisterPitBullOptionsMethod(function(group)
	if group ~= 'player' then
		return
	end
	return {
		type = 'group',
		name = L["Spark"],
		desc = L["Spark"],
		args = {
			toggle = {
				type = "toggle",
				name = L["Show Power Spark"],
				desc = L["Show the spark that displays the mana"],
				get = enabled_get,
				set = enabled_set,
			},
			color = {
				type = "color",
				name = L["Power Spark Color"],
				desc = L["Set the color of the power spark"],
				get = color_get,
				set = color_set,
				hasAlpha = true,
			},
		},
	}
end)

